#!/usr/bin/perl

###############################################################################
#                                                                             #
#               FileMail V4.2b - Mass Mailing System -                        #
#                                                                             #
#               All rights reserved. Copyright (C) 2004-2019                  #
#               Dipl.-Ing. Markus Robert Keler                               #
#               Bad Schussenried, Germany                                     #
#                                                                             #
#               For latest updates see http://developer.dipl-ing-kessler.de   #
#                                                                             #
###############################################################################
#                                                                             #
#               Purpose:                                                      #
#                                                                             #
#               Send emails to a given list of email addresses,               #
#               based on a text and/or html file which can be created using   #
#               any editor and which is imported and sent during runtime.     #
#                                                                             #
###############################################################################

require             "/full/path/to/filemail/filemail.cfg";

$mailprog         = '/usr/sbin/sendmail';

### !!! DO NOT CHANGE ANY SETTINGS BELOW !!! ##################################

if($mail_type ne 'text' && $mail_type ne 'html' && $mail_type ne 'text/html') {
   # Print Error Message
   print "Email type out of range.\nMsg has not been sent.\nExiting.\n\n";
   exit;
}

### Write to logfile ##########################################################

sub header_logfile {
   open(LogFile,">>$logfile") or die "Cannot write to logfile. Status: $!\n";
   print LogFile "\n\n/**************************************",
      "***************************************/\n\n",
      "Subject: $subject\n\n";
   close(LogFile);
}

sub entry_logfile {
   my($status, $cause)= @_;
   ( $sec, $min, $hour, $day, $mon_0, $year_1900, $wday, $yday, $isdst ) =
      localtime(time);
   $year  = $year_1900 + 1900;
   $month = $mon_0 + 1;
   my $timestamp =
      sprintf ("%02d.%02d.%4d %02d:%02d:%02d", $day, $month, $year, $hour, $min, $sec);

   open(LogFile,">>$logfile") or die "Cannot write to logfile. Status: $!\n";
   print LogFile "$timestamp $status - $cause\n";

   close(LogFile);
}

### Load email addresses ######################################################

if ($unique_yes_no eq 'yes') {
   open(EmailFile,$to_file) or die "Couldn't open To_file: $!\n";
   @lines=<EmailFile>;
   close(EmailFile);
}

else {
   open(EmailFile,$to_file) or die "Couldn't open To_file: $!\n";
   @line=<EmailFile>;
   close(EmailFile);
   $ToField='';
   foreach $i (@line) {
      chomp($i);
      $ToField .= "$i,";
   }
   close(EmailFile);

   open(EmailFile,$bcc_file) or die "Couldn't open Bcc_file: $!\n";
   @line=<EmailFile>;
   close(EmailFile);
   $BccField='';
   foreach $i (@line) {
      chomp($i);
      $BccField .= "$i,";
   }
   close(EmailFile);
}

### Load text / html body #####################################################
if($mail_type eq 'text' || $mail_type eq 'text/html') {
   open(MailBodyFile,$textbody_text) or die "Couldn't open Textbody_text_file: $!\n";
   @MailBodyLines_Text=<MailBodyFile>;
   close(MailBodyFile);
   $MailBody_Text='';
   foreach $i (@MailBodyLines_Text) {
      $MailBody_Text .= $i;
   }
}
if($mail_type eq 'html' || $mail_type eq 'text/html') {
   open(MailBodyFile,$textbody_html) or die "Couldn't open Textbody_html_file: $!\n";
   @MailBodyLines_Html=<MailBodyFile>;
   close(TextBodyFile);
   $MailBody_Html='';
   foreach $i (@MailBodyLines_Html) {
      $MailBody_Html .= $i;
   }
}


### Open Mail #################################################################

print "\nAttempting to send email...\n\n";

if ($unique_yes_no eq 'yes') {

   ### Print to File "Mail" : Unique Mail #####################################

   header_logfile();

   foreach $current_line (@lines) {

      $MailBody_Text_LocalCopy = $MailBody_Text;
      $MailBody_Html_LocalCopy = $MailBody_Html;
      $Subject_LocalCopy       = $subject;

      @fields = split(/$field_separator/,$current_line);

      ### Full name and email address in which column? ###
      $rcpt = "\"$fields[$name_in_column]\" <$fields[$mail_in_column]>";

      ### Write error to logfile, next recipient ###
      if ( ($rcpt =~ tr/@//) < 1 or ($rcpt =~ tr/.//) < 1 ){
         print "Skipping: $rcpt\n";
         entry_logfile("FAILED!!", "Mail to $rcpt.");
         next;
      }

      $max_fields = $#fields;
      if ($personal_yes_no eq 'yes') {
         for ($index=0; $index<=$max_fields; $index++) {
            $substitution = $fields[$index]; chomp($substitution);
            $Subject_LocalCopy       =~ s/\[\[$index\]\]/$substitution/g;
            $MailBody_Text_LocalCopy =~ s/\[\[$index\]\]/$substitution/g;
            $MailBody_Html_LocalCopy =~ s/\[\[$index\]\]/$substitution/g;
         }
      }
      open (MAIL, "|$mailprog -t") || die "Can't open $mailprog!\n";
      print MAIL "From: $from\n";
      print MAIL "Reply-to: $from\n";
      print MAIL "Return-Path: $from\n";
      print MAIL "To: $rcpt\n";
      if ($permanent_bcc ne '') {
         print MAIL "Bcc: $permanent_bcc\n";
      }
      print MAIL "X-Mailer: $X_Mailer\n";
      print MAIL "X-Priority: $priority\n";
      if ($confirm_read_del eq 'yes') {
         print MAIL "X-Confirm-Reading-To: $conf_read_mail\n";
         print MAIL "Disposition-Notification-To: $conf_del_mail\n";
      }
      print MAIL "Subject: $Subject_LocalCopy\n";
      print MAIL "MIME-Version: 1.0\n";

      if($mail_type eq 'text') {
         print MAIL "Content-Type: text/plain; charset=\"$charset\"\n\n";
         print MAIL $MailBody_Text_LocalCopy;
      }
      elsif($mail_type eq 'text/html') {
         print MAIL "Content-Type: multipart/alternative\; boundary=boundary42\n\n";
         print MAIL "--boundary42\n";
         print MAIL "Content-Type: text/plain; charset=\"$charset\"\n\n";
         print MAIL $MailBody_Text_LocalCopy;
         print MAIL "\n";
         print MAIL "--boundary42\n";
         print MAIL "Content-Type: text/html; charset=\"$charset\"\n\n";
         print MAIL $MailBody_Html_LocalCopy;
      }
      elsif($mail_type eq 'html') {
         print MAIL "Content-Type: text/html; charset=\"$charset\"\n\n";
         print MAIL $MailBody_Html_LocalCopy;
      }
      print MAIL "\n";
      close (MAIL);

      print "To: $rcpt\n";

      ### Write to logfile ###
      entry_logfile("SUCCESS ", "Mail to $rcpt.");
   }
}

else {

   ### Print to File "Mail" : One Pass ########################################

   open (MAIL, "|$mailprog -t") || die "Can't open $mailprog!\n";
   print MAIL "From: $from\n";
   print MAIL "Reply-to: $from\n";
   print MAIL "Return-Path: $from\n";
   print MAIL "To: $ToField\n";
   print MAIL "Bcc: $BccField\n";
   print MAIL "X-Mailer: $X_Mailer\n";
   print MAIL "X-Priority: $priority\n";
      if ($confirm_read_del eq 'yes') {
         print MAIL "X-Confirm-Reading-To: $conf_read_mail\n";
         print MAIL "Disposition-Notification-To: $conf_del_mail\n";
      }
   print MAIL "Subject: $subject\n";
   print MAIL "MIME-Version: 1.0\n";

   if($mail_type eq 'text') {
      print MAIL "Content-Type: text/plain; charset=\"$charset\"\n\n";
      print MAIL $MailBody_Text;
   }
   elsif($mail_type eq 'text/html') {
      print MAIL "Content-Type: multipart/alternative\; boundary=boundary42\n\n";
      print MAIL "--boundary42\n";
      print MAIL "Content-Type: text/plain; charset=\"$charset\"\n\n";
      print MAIL $MailBody_Text;
      print MAIL "\n";
      print MAIL "--boundary42\n";
      print MAIL "Content-Type: text/html; charset=\"$charset\"\n\n";
      print MAIL $MailBody_Html;
   }
   elsif($mail_type eq 'html') {
      print MAIL "Content-Type: text/html; charset=\"$charset\"\n\n";
      print MAIL $MailBody_Html;
   }
   print MAIL "\n";
   close (MAIL);

   # Print Confirmation of delivery
   print "Message has been sent...\n";
   print "To:  $ToField\n";
   print "Bcc: $BccField\n\n";
}

print "\nClosing File MAIL.\nProcess ending successfully.\nOK.\n\n";

exit;

